import os from 'node:os';
import run from './run.js';
import getCmakeParameters from './getCmakeParameters.js';
import state from '../state/index.js';

const cpuCount = os.cpus().length - 1;

export default function createLib(platform, fileType, options = {}) {
    if (!platform || !options || typeof options !== 'object' || Array.isArray(options)) {
        throw new Error('invalid platform or options');
    }

    const basePlatform = platform.split('-', 1)[0];
    if (basePlatform === 'iOS' && process.platform !== 'darwin') {
        return;
    }

    const cmakeParams = getCmakeParameters(platform, options);
    const buildType = options.isProd ? 'Release' : 'Debug';
    const platformPrefix = `${fileType ? `${fileType}-` : ''}${buildType}`;
    const libdir = `${state.config.paths.build}/${platformPrefix}/prebuilt/${platform}`;
    console.log(`${platform} is compiling...`);
    const t0 = performance.now();
    if (!options.bypassCmake) {
        run(null, [
            basePlatform === 'iOS' ? 'ios-cmake' : 'cmake', state.config.paths.cmakeDir,
            `-DCMAKE_BUILD_TYPE=${buildType}`,
            `-DCMAKE_INSTALL_PREFIX=${libdir}`,
            ...cmakeParams,
        ], platformPrefix, platform);
    }
    const t1 = performance.now();
    if (basePlatform === 'iOS') {
        run(null, ['ios-cmake', '--build', '.', '--config', buildType, '--target', 'install'], platformPrefix, platform);
    } else {
        run(null, ['make', `-j${cpuCount}`, 'install'], platformPrefix, platform);
    }
    const t2 = performance.now();
    console.log(`${platform} compiled`, platformPrefix, `full time: ${Math.round(t2 - t0)}ms`, `cmake: ${Math.round(t1 - t0)}ms`, `build: ${Math.round(t2 - t1)}ms`, `core: ${cpuCount}`);
}
