import loadJson from '../utils/loadJson.js';
import writeJson from '../utils/writeJson.js';
import loadConfig from './loadConfig.js';

const cacheDir = `${process.cwd()}/.cppjs`;

const state = {
    platforms: {
        All: ['Emscripten-x86_64', 'Android-arm64-v8a', 'iOS-iphoneos', 'iOS-iphonesimulator'],
        WebAssembly: ['Emscripten-x86_64'],
        Android: ['Android-arm64-v8a'],
        iOS: ['iOS-iphoneos', 'iOS-iphonesimulator'],
    },
    config: null,
    cache: {
        hashes: {},
        interfaces: {},
        bridges: {},
    },
};

if (typeof module !== 'undefined' && module.exports) {
    initProcessState();
} else {
    await initProcessState();
}

await initProcessState();

async function initProcessState() {
    state.cache = loadCacheState();
    state.config = await loadConfig();
}

function loadCacheState() {
    const stateFilePath = `${cacheDir}/cache.json`;
    return loadJson(stateFilePath) || state.cache;
}

export function saveCache() {
    writeJson(`${cacheDir}/cache.json`, state.cache);
}

export default state;
